      DOUBLE PRECISION FUNCTION D1MACH (I)
C***PURPOSE  Return floating point machine dependent constants.
C***DESCRIPTION
C
C   D1MACH can be used to obtain machine-dependent parameters for the
C   local machine environment.  It is a function subprogram with one
C   (input) argument, and can be referenced as follows:
C
C        D = D1MACH(I)
C
C   where I=1,...,5.  The (output) value of D above is determined by
C   the (input) value of I.  The results for various values of I are
C   discussed below.
C
C   D1MACH( 1) = B**(EMIN-1), the smallest positive magnitude.
C   D1MACH( 2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
C   D1MACH( 3) = B**(-T), the smallest relative spacing.
C   D1MACH( 4) = B**(1-T), the largest relative spacing.
C   D1MACH( 5) = LOG10(B)
C
C   Those values are taken from DLAMCH.
C
      DOUBLE PRECISION DMACH(5)
      INTEGER I
      LOGICAL INIT
      SAVE INIT, DMACH
      DATA INIT /.FALSE./
C
      IF (.NOT. INIT) THEN
          DMACH(1) = DLAMCH('U')
          DMACH(2) = DLAMCH('O')
          DMACH(3) = DLAMCH('E')
          DMACH(4) = DLAMCH('P')
          DMACH(5) = LOG10(DLAMCH('B'))
          INIT = .TRUE.
      ENDIF
C
C      IF (I .LT. 1 .OR. I .GT. 5) CALL XERMSG ('SLATEC', 'D1MACH',
C     +   'I OUT OF BOUNDS', 1, 2)
C
      D1MACH = DMACH(I)
      RETURN
C
      END
